@@ -87,7 +87,7 @@ int wcmIsDuplicate(char* device, LocalDevicePtr local)
 	/* always allow xorg.conf defined tools to be added */
 	if (!strlen(lsource)) goto ret;
 
-	if (fstat(local->fd, &st) == -1)
+	if (stat(device, &st) == -1)
 	{
 		/* can not access major/minor to check device duplication */
 		xf86Msg(X_ERROR, "%s: stat failed (%s). cannot check for duplicates.\n",
@@ -168,11 +168,9 @@ int wcmDeviceTypeKeys(LocalDevicePtr local)
 {
 	int ret = 1;
 	WacomDevicePtr priv = local->private;
+	WacomCommonPtr common = priv->common;
 
-	/* serial ISDV4 devices */
-	priv->common->tablet_id = isdv4ProbeKeys(local);
-	if (!priv->common->tablet_id) /* USB devices */
-		priv->common->tablet_id = usbProbeKeys(local);
+	priv->common->tablet_id = common->wcmDevCls->ProbeKeys(local);
 
 	switch (priv->common->tablet_id)
 	{
@@ -267,8 +265,7 @@ static InputOption *wcmOptionDupConvert(LocalDevicePtr local, const char* basena
 	memset(&dummy, 0, sizeof(dummy));
 	xf86CollectInputOptions(&dummy, NULL, original);
 
-	name = calloc(strlen(basename) + strlen(type) + 2, 1);
-	sprintf(name, "%s %s", basename, type);
+	name = Xprintf("%s %s", basename, type);
 
 	dummy.options = xf86ReplaceStrOption(dummy.options, "Type", type);
 	dummy.options = xf86ReplaceStrOption(dummy.options, "Name", name);
@@ -311,11 +308,7 @@ static InputAttributes* wcmDuplicateAttributes(LocalDevicePtr local,
 {
 	InputAttributes *attr;
 	attr = DuplicateInputAttributes(local->attrs);
-							/* add one space, one \0 */
-	attr->product = realloc(attr->product, strlen(attr->product) + strlen(type) + 2);
-	strcat(attr->product, " ");
-	strcat(attr->product, type);
-
+	attr->product = Xprintf("%s %s", attr->product, type);
 	return attr;
 }
 #endif
@@ -688,31 +681,19 @@ int wcmParseOptions(LocalDevicePtr local, int hotplugged)
 	if ( !priv->wcmMMonitor )
 		priv->wcmMMonitor = xf86SetBoolOption(local->options, "MMonitor", 1);
 
+	/* Swap stylus buttons 2 and 3 for Tablet PCs */
+	if (TabletHasFeature(common, WCM_TPC) && IsStylus(priv))
+	{
+		priv->button[1] = 3;
+		priv->button[2] = 2;
+	}
+
 	for (i=0; i<WCM_MAX_BUTTONS; i++)
 	{
 		sprintf(b, "Button%d", i+1);
 		priv->button[i] = xf86SetIntOption(local->options, b, priv->button[i]);
 	}
 
-	if (common->wcmForceDevice == DEVICE_ISDV4)
-        {
-		int val;
-		val = xf86SetIntOption(local->options, "BaudRate", 38400);
-
-		switch(val)
-		{
-			case 38400:
-			case 19200:
-				common->wcmISDV4Speed = val;
-				break;
-			default:
-				xf86Msg(X_ERROR, "%s: Illegal speed value "
-					"(must be 19200 or 38400).",
-					local->name);
-				break;
-		}
-	}
-
 	s = xf86SetStrOption(local->options, "Twinview", NULL);
 	if (s && xf86NameCmp(s, "none") == 0)
 		priv->twinview = TV_NONE;
@@ -739,6 +720,11 @@ int wcmParseOptions(LocalDevicePtr local, int hotplugged)
 	else
 		priv->numScreen = screenInfo.numScreens;
 
+	/* Now parse class-specific options */
+	if (common->wcmDevCls->ParseOptions &&
+	    !common->wcmDevCls->ParseOptions(local))
+		goto error;
+
 	return 1;
 error:
 	free(area);
@@ -746,21 +732,4 @@ error:
 	return 0;
 }
 
-int wcmAutoProbeDevice(LocalDevicePtr local)
-{
-	WacomDevicePtr priv = (WacomDevicePtr) local->private;
-	WacomCommonPtr common =  priv->common;
-
-	if ((!common->wcmDevice || !strcmp (common->wcmDevice, "auto-dev")))
-	{
-		common->wcmFlags |= AUTODEV_FLAG;
-		if (! (common->wcmDevice = wcmEventAutoDevProbe (local)))
-		{
-			xf86Msg(X_ERROR, "%s: unable to probe device\n",
-				local->name);
-			return 0;
-		}
-	}
-	return 1;
-}
 /* vim: set noexpandtab shiftwidth=8: */
