@@ -42,7 +42,6 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
-#include <linux/serial.h>
 
 #include "xf86Wacom.h"
 #include <xf86_OSproc.h>
@@ -631,35 +630,31 @@ static int wcmRegisterX11Devices (LocalDevicePtr local)
 	}
 
 
-	/* only initial KeyClass and LedFeedbackClass once */
-	if (!priv->wcmInitKeyClassCount)
-	{
 #if GET_ABI_MAJOR(ABI_XINPUT_VERSION) >= 7
-		if (InitKeyboardDeviceStruct(local->dev, NULL, NULL, wcmKbdCtrlCallback)) {
+	if (InitKeyboardDeviceStruct(local->dev, NULL, NULL, wcmKbdCtrlCallback)) {
 #define SYMS_PER_KEY 2
-			KeySymsRec syms;
-			CARD8 modmap[MAP_LENGTH];
-			int num_keys = XkbMaxLegalKeyCode - XkbMinLegalKeyCode + 1;
+		KeySymsRec syms;
+		CARD8 modmap[MAP_LENGTH];
+		int num_keys = XkbMaxLegalKeyCode - XkbMinLegalKeyCode + 1;
 
-			syms.map = keymap;
-			syms.mapWidth = SYMS_PER_KEY;
-			syms.minKeyCode = XkbMinLegalKeyCode;
-			syms.maxKeyCode = XkbMaxLegalKeyCode;
+		syms.map = keymap;
+		syms.mapWidth = SYMS_PER_KEY;
+		syms.minKeyCode = XkbMinLegalKeyCode;
+		syms.maxKeyCode = XkbMaxLegalKeyCode;
 
-			memset(modmap, 0, sizeof(modmap));
-			modmap[XkbMinLegalKeyCode + 2] = ShiftMask;
-			XkbApplyMappingChange(local->dev, &syms, syms.minKeyCode, num_keys, NULL, // modmap,
-					serverClient);
-		} else
-		{
-			xf86Msg(X_ERROR, "%s: unable to init kbd device struct\n", local->name);
-			return FALSE;
-		}
+		memset(modmap, 0, sizeof(modmap));
+		modmap[XkbMinLegalKeyCode + 2] = ShiftMask;
+		XkbApplyMappingChange(local->dev, &syms, syms.minKeyCode, num_keys, NULL, // modmap,
+				serverClient);
+	} else
+	{
+		xf86Msg(X_ERROR, "%s: unable to init kbd device struct\n", local->name);
+		return FALSE;
+	}
 #endif
-		if(InitLedFeedbackClassDeviceStruct (local->dev, wcmKbdLedCallback) == FALSE) {
-			xf86Msg(X_ERROR, "%s: unable to init led feedback device struct\n", local->name);
-			return FALSE;
-		}
+	if(InitLedFeedbackClassDeviceStruct (local->dev, wcmKbdLedCallback) == FALSE) {
+		xf86Msg(X_ERROR, "%s: unable to init led feedback device struct\n", local->name);
+		return FALSE;
 	}
 
 	wcmInitialToolSize(local);
@@ -725,29 +720,23 @@ static int wcmRegisterX11Devices (LocalDevicePtr local)
 				-64, 63, 1, 1, 1);
 	}
 
-	if ((TabletHasFeature(common, WCM_ROTATION)) && IsStylus(priv))
-		/* Art Marker Pen rotation */
-		InitValuatorAxisStruct(local->dev, 5,
-#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) >= 7
-				0, /* XXX what is this axis?*/
-#endif
-				-900, 899, 1, 1, 1);
-	else if ((TabletHasFeature(common, WCM_RING)) && IsPad(priv))
-		/* Touch ring */
-		InitValuatorAxisStruct(local->dev, 5,
-#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) >= 7
-				0, /* XXX what is this axis?*/
-#endif
-				0, 71, 1, 1, 1);
-	else
+	if (IsStylus(priv))
 	{
-		/* absolute wheel */
+		int maxRotation = MAX_ROTATION_RANGE + MIN_ROTATION - 1;
+		/* Art Marker Pen rotation or Airbrush absolute Wheel */
 		InitValuatorAxisStruct(local->dev, 5,
 #if GET_ABI_MAJOR(ABI_XINPUT_VERSION) >= 7
 				XIGetKnownProperty(AXIS_LABEL_PROP_ABS_WHEEL),
 #endif
-				0, 1023, 1, 1, 1);
+				MIN_ROTATION, maxRotation, 1, 1, 1);
 	}
+	else if ((TabletHasFeature(common, WCM_RING)) && IsPad(priv))
+		/* Touch ring */
+		InitValuatorAxisStruct(local->dev, 5,
+#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) >= 7
+				XIGetKnownProperty(AXIS_LABEL_PROP_ABS_WHEEL),
+#endif
+				0, 71, 1, 1, 1);
 
 	if (IsTouch(priv))
 	{
@@ -826,6 +815,7 @@ char *wcmEventAutoDevProbe (LocalDevicePtr local)
 	}
 	xf86Msg(X_ERROR, "%s: no Wacom event device found (checked %d nodes, waited %d msec)\n",
 		local->name, i + 1, wait);
+	xf86Msg(X_ERROR, "%s: unable to probe device\n", local->name);
 	return FALSE;
 }
 
@@ -833,14 +823,10 @@ char *wcmEventAutoDevProbe (LocalDevicePtr local)
  * wcmOpen --
  ****************************************************************************/
 
-static Bool wcmOpen(LocalDevicePtr local)
+Bool wcmOpen(LocalDevicePtr local)
 {
 	WacomDevicePtr priv = (WacomDevicePtr)local->private;
 	WacomCommonPtr common = priv->common;
-	char id[BUFFER_SIZE];
-	float version;
-	int rc;
-	struct serial_struct ser;
 
 	DBG(1, priv, "opening device file\n");
 
@@ -848,44 +834,20 @@ static Bool wcmOpen(LocalDevicePtr local)
 	if (local->fd < 0)
 	{
 		xf86Msg(X_ERROR, "%s: Error opening %s (%s)\n", local->name,
-			common->wcmDevice, strerror(errno));
+			common->device_path, strerror(errno));
 		return !Success;
 	}
 
-	rc = ioctl(local->fd, TIOCGSERIAL, &ser);
-
-	/* we initialized wcmDeviceClasses to USB
-	 * Bluetooth is also considered as USB */
-	if (rc == 0) /* serial device */
-	{
-		/* only ISDV4 are supported on X server 1.7 and later */
-		common->wcmForceDevice=DEVICE_ISDV4;
-		common->wcmDevCls = &gWacomISDV4Device;
-	}
-	else
-	{
-		/* Detect USB device class */
-		if ((&gWacomUSBDevice)->Detect(local))
-			common->wcmDevCls = &gWacomUSBDevice;
-		else
-		{
-			xf86Msg(X_ERROR, "%s: wcmOpen found undetectable "
-				" %s \n", local->name, common->wcmDevice);
-			return !Success;
-		}
-	}
-
-	/* Initialize the tablet */
-	if(common->wcmDevCls->Init(local, id, &version) != Success ||
-		wcmInitTablet(local, id, version) != Success)
-	{
-		xf86CloseSerial(local->fd);
-		local->fd = -1;
-		return !Success;
-	}
 	return Success;
 }
 
+static int wcmDevInit(DeviceIntPtr pWcm)
+{
+	LocalDevicePtr local = (LocalDevicePtr)pWcm->public.devicePrivate;
+
+	return wcmRegisterX11Devices (local);
+}
+
 /*****************************************************************************
  * wcmDevOpen --
  *    Open the physical device and init information structs.
@@ -896,27 +858,17 @@ static int wcmDevOpen(DeviceIntPtr pWcm)
 	LocalDevicePtr local = (LocalDevicePtr)pWcm->public.devicePrivate;
 	WacomDevicePtr priv = (WacomDevicePtr)local->private;
 	WacomCommonPtr common = priv->common;
+	WacomModelPtr model = common->wcmModel;
 	struct stat st;
 
 	DBG(10, priv, "\n");
 
-	/* Device has been open and not autoprobed */
-	if (priv->wcmDevOpenCount)
-		return TRUE;
-
 	/* open file, if not already open */
 	if (common->fd_refs == 0)
 	{
-		/* Autoprobe if necessary */
-		if ((common->wcmFlags & AUTODEV_FLAG) &&
-		    !(common->wcmDevice = wcmEventAutoDevProbe (local)))
-			xf86Msg(X_ERROR, "%s: Cannot probe device\n", local->name);
-
-		if ((wcmOpen (local) != Success) || (local->fd < 0) ||
-			!common->wcmDevice)
+		if ((wcmOpen (local) != Success) || !common->device_path)
 		{
-			DBG(1, priv, "Failed to open "
-				"device (fd=%d)\n", local->fd);
+			DBG(1, priv, "Failed to open device (fd=%d)\n", local->fd);
 			if (local->fd >= 0)
 			{
 				DBG(1, priv, "Closing device\n");
@@ -929,8 +881,7 @@ static int wcmDevOpen(DeviceIntPtr pWcm)
 		if (fstat(local->fd, &st) == -1)
 		{
 			/* can not access major/minor */
-			DBG(1, priv, "stat failed (%s). "
-				"cannot check status.\n", strerror(errno));
+			DBG(1, priv, "stat failed (%s).\n", strerror(errno));
 
 			/* older systems don't support the required ioctl.
 			 * So, we have to let it pass */
@@ -949,8 +900,9 @@ static int wcmDevOpen(DeviceIntPtr pWcm)
 		common->fd_refs++;
 	}
 
-	if (!wcmRegisterX11Devices (local))
-		return FALSE;
+	/* start the tablet data */
+	if (model->Start && (model->Start(local) != Success))
+		return !Success;
 
 	return TRUE;
 }
@@ -1012,8 +964,7 @@ void wcmReadPacket(LocalDevicePtr local)
 
 	remaining = sizeof(common->buffer) - common->bufpos;
 
-	DBG(1, common, "pos=%d"
-		" remaining=%d\n", common->bufpos, remaining);
+	DBG(1, common, "pos=%d remaining=%d\n", common->bufpos, remaining);
 
 	/* fill buffer with as much data as we can handle */
 	len = xf86ReadSerial(local->fd,
@@ -1122,7 +1073,9 @@ static void wcmDevClose(LocalDevicePtr local)
 static int wcmDevProc(DeviceIntPtr pWcm, int what)
 {
 	LocalDevicePtr local = (LocalDevicePtr)pWcm->public.devicePrivate;
+#ifdef DEBUG
 	WacomDevicePtr priv = (WacomDevicePtr)local->private;
+#endif
 	Status rc = !Success;
 
 	DBG(2, priv, "BEGIN dev=%p priv=%p "
@@ -1140,18 +1093,13 @@ static int wcmDevProc(DeviceIntPtr pWcm, int what)
 	switch (what)
 	{
 		case DEVICE_INIT:
-			priv->wcmDevOpenCount = 0;
-			priv->wcmInitKeyClassCount = 0;
-			if (!wcmDevOpen(pWcm))
+			if (!wcmDevInit(pWcm))
 				goto out;
-			priv->wcmInitKeyClassCount++;
-			priv->wcmDevOpenCount++;
-			break; 
+			break;
 
 		case DEVICE_ON:
 			if (!wcmDevOpen(pWcm))
 				goto out;
-			priv->wcmDevOpenCount++;
 			xf86AddEnabledDevice(local);
 			pWcm->public.on = TRUE;
 			break;
@@ -1164,7 +1112,6 @@ static int wcmDevProc(DeviceIntPtr pWcm, int what)
 				wcmDevClose(local);
 			}
 			pWcm->public.on = FALSE;
-			priv->wcmDevOpenCount = 0;
 			break;
 
 		default:
