@@ -113,6 +113,9 @@ struct _WacomModule
  */
 #define SYSCALL(call) while(((call) == -1) && (errno == EINTR))
 
+/* Open the device with the right serial parmeters */
+extern Bool wcmOpen(LocalDevicePtr local);
+
 /* device autoprobing */
 char *wcmEventAutoDevProbe (LocalDevicePtr local);
 
@@ -144,7 +147,6 @@ extern int wcmDeviceTypeKeys(LocalDevicePtr local);
 /* hotplug */
 extern int wcmNeedAutoHotplug(LocalDevicePtr local, const char **type);
 extern void wcmHotplugOthers(LocalDevicePtr local, const char *basename);
-extern int wcmAutoProbeDevice(LocalDevicePtr local);
 
 /* setup */
 extern int wcmParseOptions(LocalDevicePtr local, int hotplugged);
@@ -159,7 +161,7 @@ extern int wcmDevSwitchMode(ClientPtr client, DeviceIntPtr dev, int mode);
 extern void wcmChangeScreen(LocalDevicePtr local, int value);
 extern void wcmTilt2R(WacomDeviceStatePtr ds);
 extern void wcmGestureFilter(WacomDevicePtr priv, int channel);
-extern void wcmEmitKeysym(DeviceIntPtr keydev, int keysym, int state);
+extern void wcmEmitKeycode(DeviceIntPtr keydev, int keycode, int state);
 extern void wcmSoftOutEvent(LocalDevicePtr local);
 
 extern void wcmRotateTablet(LocalDevicePtr local, int value);
@@ -174,13 +176,14 @@ extern int wcmGetPhyDeviceID(WacomDevicePtr priv);
 extern int wcmSetProperty(DeviceIntPtr dev, Atom property, XIPropertyValuePtr prop, BOOL checkonly);
 extern void InitWcmDeviceProperties(LocalDevicePtr local);
 
-/* Device probing */
-int isdv4ProbeKeys(LocalDevicePtr local);
-int usbProbeKeys(LocalDevicePtr local);
-
 /* Utility functions */
 extern Bool is_absolute(LocalDevicePtr local);
 extern void set_absolute(LocalDevicePtr local, Bool absolute);
+extern WacomCommonPtr wcmRefCommon(WacomCommonPtr common);
+extern void wcmFreeCommon(WacomCommonPtr *common);
+extern WacomCommonPtr wcmNewCommon(void);
 
 /****************************************************************************/
 #endif /* __XF86WACOM_H */
+
+/* vim: set noexpandtab shiftwidth=8: */
